#include "lights.hpp"
#include "constants.hpp"
#include <vector>

using std::vector;

namespace lighting {
  void LightRender::render_square_light(LightSource source, Point at, PointList &has_light) {
    for(matrix::in_box it{$lightmap, at.x, at.y, (size_t)floor(source.radius)}; it.next();) {
      if($paths.$paths[it.y][it.x] != WALL_PATH_LIMIT) {
        $lightmap[it.y][it.x] = light_level(source.strength, it.distance(), it.x, it.y);
        has_light.push_back({it.x, it.y});
      }
    }
  }

  void LightRender::render_light(LightSource source, Point at) {
    Point min, max;
    clear_light_target(at);
    PointList has_light;

    render_square_light(source, at, has_light);

    const int wall_light = source.strength + WALL_LIGHT_LEVEL;
    for(auto point : has_light) {
      for(matrix::compass it{$lightmap, point.x, point.y}; it.next();) {
        if($paths.$paths[it.y][it.x] == WALL_PATH_LIMIT) {
            // BUG: include the distance in the list of walls to light
            // so that they will be closer to the light level at that point
            $lightmap[it.y][it.x] = light_level(wall_light, 1.0f, point.x, point.y);
        }
      }
    }
  }

  int LightRender::light_level(int strength, float distance, size_t x, size_t y) {
    int cur_level = $lightmap[y][x];
    int new_level = strength / sqrt(distance + 0.6f);

    return cur_level < new_level ? new_level : cur_level;
  }


  void LightRender::reset_light() {
    for(auto &row : $lightmap) {
      row.assign(row.size(), lighting::MIN);
    }
  }

  void LightRender::clear_light_target(const Point &at) {
    $paths.clear_target(at);
  }

  void LightRender::set_light_target(const Point &at, int value) {
    $paths.set_target(at, value);
  }

  void LightRender::path_light(Matrix &walls) {
    $paths.compute_paths(walls);
  }
}
